`timescale 1ns / 1ps
module fast_power_dp(
  input CLK, RESET, LOADPOW, MULPOW,
        LOADPOW2N, LOADEXP, RDONE, SDONE,
  input [7:0] EXP, BASE,
  output EXP_ZERO_N, EXP_BIT,
  output reg DONE,
  output reg [511:0] POW
);

  reg [7:0] EXP_REG; // Exponentenregister, desktruktiv
  reg [511:0] POW2N; // (2**n)-te Potenzen von BASE

  // Statussignale fuer Steuerwerk
  // Exponentenregister ist nicht Null
  assign EXP_ZERO_N = |EXP_REG;
  // Aktuelles Exponentenbit
  assign EXP_BIT = EXP_REG[0];

  always @(posedge CLK or posedge RESET) begin
    if (RESET) begin
      DONE <= 0;
      POW <= 0;
      EXP_REG <= 0;
      POW2N <= 0;
    end
    else begin
      // DONE behandeln
      case ({SDONE, RDONE})
        2'b10: DONE <= 1;
        2'b01: DONE <= 0;
      endcase
      // EXP_REG behandeln
      if (LOADEXP)
        EXP_REG <= EXP;
      else // Exponent 1 Bit nach rechts schieben
        EXP_REG <= {1'b0, EXP_REG[7:1]};
      // (2**n)-te Potenzen der Basis berechnen
      if (LOADPOW2N)
        POW2N <= BASE;
      else
        POW2N <= POW2N * POW2N;
      // Ergebnisakkumulator POW behandeln
      if (LOADPOW)
        POW <= 1;
      else
        if (MULPOW)
          POW <= POW * POW2N;
    end
  end

endmodule
