`timescale 1ns / 1ps
module fast_power_fsm(
  input CLK, RESET, START, EXP_ZERO_N, EXP_BIT,
  output reg LOADPOW, MULPOW, LOADPOW2N,
             LOADEXP, RDONE, SDONE
);

  parameter IDLE = 0,
            S0   = 1,
            S1   = 2,
            S2   = 3;

  reg [1:0] STATE, NEXTSTATE;
  
  // Kombinatorisch neuen Zustand und Ausgangsvektor berechnen
  // MULPOW haengt direkt von EXP_BIT ab => Mealy-Automat
  always @(STATE, START, EXP_ZERO_N, EXP_BIT) begin
    // Default Werte, Latches vermeiden
    // alles deaktivieren
    LOADPOW = 0; MULPOW = 0; LOADPOW2N = 0;
    LOADEXP = 0; RDONE = 0; SDONE = 0;
    NEXTSTATE = IDLE;
    case (STATE)
      IDLE: if (START)            // Auf Startsignal warten
              NEXTSTATE = S0;
      S0:   begin                 // Datenpfad initialisieren
              LOADPOW = 1; LOADPOW2N = 1;
              LOADEXP = 1; RDONE = 1;
              NEXTSTATE = S1;
            end
      S1:   if (EXP_ZERO_N) begin // Schleife EXP != 0
              if (EXP_BIT)
                MULPOW = 1;
              NEXTSTATE = S1;
            end
            else
              NEXTSTATE = S2;
      S2:   begin                 // Ende der Berechnung anzeigen
              SDONE = 1;
              NEXTSTATE = IDLE;
            end
    endcase
  end

  // Neuen Zustand uebernehmen
  always @(posedge CLK, posedge RESET) begin
    if (RESET)  STATE <= IDLE;
    else        STATE <= NEXTSTATE;
  end

endmodule
